/*
  This is a gateway node between mysensors and 433Mhz devices. A 433Mhz receiver is connected to pin 2 (int 0). A sender is connected to pin 7
  This works with any 433Mhz device (as long as rc-switch understands the protocol).

  For the sender and the receiver the sketch registers a text sensor each. Incoming messages via 433Mhz are send to the controller via the 
  receiver text sensor (int codes are send). If the controller decides to answer it can send a message with the same code (or any other)
  to the sender sensor. These codes are extracted from the message and send via 433Mhz.

  Still work in progresss!

  This is based on the MySensors network: https://www.mysensors.org
  This uses the rc-switch library to detect button presses: https://github.com/sui77/rc-switch

  Written by Oliver Hilsky 28.06.2017
*/

// define radio stuff
#define MY_RADIO_NRF24
#define MY_BAUD_RATE 9600
#define MY_DEBUG    // Enables debug messages in the serial log
//#define MY_NODE_ID 13

// add signing stuff
//#define MY_SIGNING_SOFT // Enables software signing
//#define MY_SIGNING_REQUEST_SIGNATURES // Always request signing from gateway
//#define MY_SIGNING_SOFT_RANDOMSEED_PIN 7 // floating pin for randomness
//#define MY_DEBUG_VERBOSE_SIGNING

#include <EEPROM.h>
#include <SPI.h>
#include <MyConfig.h>
#include <MySensors.h>
#include <Vcc.h>
#include <RCSwitch.h>

const long RECEIVER_PIN = 0;
const long SENDER_PIN = 7;
const long DEBOUNCE = 500;
const long CHILD_RECEIVER = 0;
const long CHILD_SENDER = 1;


RCSwitch wirelessSwitch = RCSwitch();


void presentation()
{
  sendSketchInfo("433 Wireless Gateway", "29062017");

  present(CHILD_RECEIVER, S_INFO); // able to receive text
  present(CHILD_SENDER, S_INFO); // able to send text
}

void setup() {
  wirelessSwitch.enableReceive(RECEIVER_PIN); // Receiver on interrupt 0 => that is pin #2
  wirelessSwitch.enableTransmit(SENDER_PIN);  // Sender on pin 7
  Serial.println("Started listening on pin 2 for incoming 433Mhz messages");
}

void loop() {
  if (wirelessSwitch.available()) {
  	// for testing
    //output(wirelessSwitch.getReceivedValue(), wirelessSwitch.getReceivedBitlength(), wirelessSwitch.getReceivedDelay(), wirelessSwitch.getReceivedRawdata(),wirelessSwitch.getReceivedProtocol());

    unsigned long received_val = wirelessSwitch.getReceivedValue();

    Serial.print("Received 433Mhz message: ");
    Serial.println(received_val);

    send(MyMessage(CHILD_RECEIVER, V_TEXT).set(received_val));

    // software debounce
    wait(DEBOUNCE);
    wirelessSwitch.resetAvailable();
  }
}

void receive(const MyMessage &message) {
  Serial.print("Sensor: "); Serial.print(message.sensor); Serial.print(", Message: "); Serial.println(message.getString()); 
  if (message.type == V_TEXT) { 
    if (message.sensor == CHILD_SENDER) {
      Serial.println("Sending message via 433Mhz");
      char * messageContent = strdup(message.getString());
      wirelessSwitch.send(messageContent);
    }
  }
}
